__all__ = ('get_cuda_info',)

from collections import namedtuple

from ..logger import LOG

Status = namedtuple('Status', 'version,gpu_availability')


def get_gpu_status_for_tf() -> dict:
    status = Status(version=None, gpu_availability=False)
    try:
        import tensorflow as tf
        LOG.info('tensorflow version = %s' % tf.__version__)
        status = Status(
            version=tf.__version__,
            gpu_availability=tf.test.is_gpu_available()
        )
    except Exception as e:
        LOG.error('Error detecting CUDA availability for tensorflow')
        LOG.error(str(e))
    return status._asdict()


def get_gpu_status_for_torch() -> dict:
    status = Status(version=None, gpu_availability=False)
    try:
        import torch
        LOG.info('torch version = %s' % torch.__version__)
        status = Status(
            version=torch.__version__,
            gpu_availability=torch.cuda.is_available()
        )
    except Exception as e:
        LOG.error('Error detecting CUDA availability for torch')
        LOG.error(str(e))
    return status._asdict()


def get_cuda_info() -> dict:
    return {
        "cuda_availability_for_torch": get_gpu_status_for_torch(),
        "cuda_availability_for_tf": get_gpu_status_for_tf(),
    }


def main(*args, **kwargs):
    data = get_cuda_info()
    import json
    print(json.dumps(data, ensure_ascii=False, indent=2))


if __name__ == '__main__':
    main()



__all__ = ('get_gpu_info',)

# ! `pip install pynvml`, reference: https://github.com/gpuopenanalytics/pynvml

from collections import namedtuple

from ..logger import LOG

try:
    import pynvml
    from pynvml import NVMLError
except ImportError:
    raise RuntimeError('Please install package `pynvml` first!')

Device = namedtuple('Device', field_names='index,name,arch')
DeviceStatus = namedtuple('DeviceStatus', field_names='mem_total,mem_free,mem_used,gpu_rate,mem_rate')


class NvInfo:
    def __init__(self):
        self.nv = pynvml

        try:
            self.nv.nvmlInit()
            ver: bytes = self.nv.nvmlSystemGetDriverVersion()
            version = ver.decode(encoding='UTF-8')
            LOG.info('Nvidia Driver version: %s' % version)
        except NVMLError as e:
            LOG.info('NVML library NOT found!')
            LOG.info(str(e))

    def __del__(self):
        try:
            self.nv.nvmlShutdown()
        except (OSError, NVMLError):
            pass

    @property
    def get_device_count(self) -> int:
        try:
            return self.nv.nvmlDeviceGetCount()
        except NVMLError:
            return 0

    def get_device_list(self) -> list:
        list_device = []
        for i in range(self.get_device_count):
            handler = self._get_device_handler(index_device=i)
            name, arch = None, None

            try:
                name = self.nv.nvmlDeviceGetName(handler).decode(encoding='UTF-8')
            except NVMLError as e:
                LOG.info('Failed to get device name!')
                LOG.info(str(e))

            try:
                arch = self.nv.nvmlDeviceGetArchitecture(handler)
            except NVMLError as e:
                LOG.info('Failed to get device architecture!')
                LOG.info(str(e))

            device = Device(index=i, name=name, arch=arch)
            LOG.info('Found device {index} info: name={name}; arch={arch}'.format(**device._asdict()))
            list_device.append(device)

        return list_device

    def _get_device_handler(self, index_device=0):
        return self.nv.nvmlDeviceGetHandleByIndex(index_device)

    def get_device_status(self, index_device=0) -> DeviceStatus:
        handler = self._get_device_handler(index_device=index_device)
        mem_info = self.nv.nvmlDeviceGetMemoryInfo(handler)
        util_info = self.nv.nvmlDeviceGetUtilizationRates(handler)
        return DeviceStatus(
            mem_total=mem_info.total,
            mem_free=mem_info.free,
            mem_used=mem_info.used,
            gpu_rate=util_info.gpu,
            mem_rate=util_info.memory
        )


def get_gpu_info() -> dict:
    nv_info = NvInfo()
    result = []
    list_devices = nv_info.get_device_list()
    for i, d in enumerate(list_devices):
        status = nv_info.get_device_status(index_device=i)
        item = {
            'device': d._asdict(),
            'status': status._asdict()
        }
        result.append(item)

    return {
        "gpu_info": result
    }


def main(*args, **kwargs):
    info_gpus = get_gpu_info()
    import json
    print(json.dumps(info_gpus, ensure_ascii=False, indent=2))


if __name__ == '__main__':
    main()



import abc
import json
import logging
from typing import Optional, Awaitable

from tornado import web

from ...logger import LOG

_RESP_BAD_REQUEST = {'code': '5101', 'message': ['Bad request: fail to parse body as JSON object!']}


class APIHandler(web.RequestHandler):
    LOG = LOG

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    @abc.abstractmethod
    def response(self, *args, **kwargs) -> dict:
        raise NotImplementedError()

    def set_default_headers(self) -> None:
        self.set_header('Content-Type', 'application/json; charset=utf-8')

    def data_received(self, chunk: bytes) -> Optional[Awaitable[None]]:
        pass

    async def post(self, *args, **kwargs):
        try:
            req = self.request.body
            body = json.loads(req.decode('utf-8'))
            kwargs.update(body)
        except json.decoder.JSONDecodeError:  # invalid request body, cannot be parsed as JSON
            return self.finish(_RESP_BAD_REQUEST)

        resp = dict(code=5200, message=['success'])
        try:
            result = self.response(*args, **kwargs)  # this call may throw TypeError when argument missing
            resp['data'] = result
        except Exception as e:
            if LOG.level == logging.DEBUG:
                self.LOG.error(e, exc_info=True)
            return self.finish({'code': 5201, 'message': [str(e)]})

        resp = json.dumps(resp, ensure_ascii=False, default=str, separators=(',', ':'))
        return self.finish(resp)



import requests

from ...logger import LOG


class APICaller:
    @staticmethod
    def wrap_request_data(data):
        assert isinstance(data, dict), "Data object must be a dict!"
        return data

    @staticmethod
    def call(api_url, timeout=5, **kwargs):
        """Trigger API call

        :param api_url: do NOT start with slash (/)
        :param timeout: requests timeout in seconds
        :param kwargs: request data
        :return:
        """
        payload = APICaller.wrap_request_data(data=kwargs)
        LOG.debug('Calling API: %s' % api_url)
        resp = requests.post(
            api_url, json=payload, timeout=timeout, headers={'Content-Type': 'application/json'}
        )

        try:
            ret = resp.json()
        except Exception as e:
            LOG.error(str(e))
            raise RuntimeError(resp.text)

        try:
            return ret['data']
        except KeyError:
            raise RuntimeError(resp.text)
